## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
# part of Unit Statistics Mod
#  by
# Teg_Navanis
# based on CvMercenaryManager.py, part of Mercenaries Mod by TheLopez
from CvPythonExtensions import *
import CvUtil
import ScreenInput
import CvScreenEnums
import PyHelpers
import UnitStatisticsUtils
import ScreenInput
import CvConfigParser
from CvStatisticsScreensEnums import *

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()
objUnitStatisticsUtils = UnitStatisticsUtils.UnitStatisticsUtils()

# Change this to either enable or disable the display of other players' unit statistics 
# Leave disabled if you don't want any additional information about enemy units
# Default value is false
g_bShowAllPlayers = false

#######SD Tool Kit#######

import SdToolKitAdvanced
sdEntityInit   = SdToolKitAdvanced.sdEntityInit
sdEntityExists = SdToolKitAdvanced.sdEntityExists
sdEntityWipe   = SdToolKitAdvanced.sdEntityWipe
sdGetVal       = SdToolKitAdvanced.sdGetVal
sdSetVal       = SdToolKitAdvanced.sdSetVal

sdObjectInit      = SdToolKitAdvanced.sdObjectInit
sdObjectWipe      = SdToolKitAdvanced.sdObjectWipe
sdObjectExists    = SdToolKitAdvanced.sdObjectExists
sdObjectGetVal    = SdToolKitAdvanced.sdObjectGetVal
sdObjectSetVal    = SdToolKitAdvanced.sdObjectSetVal

############################

class CvStatisticsScreen:
	
	def __init__(self):

		self.WIDGET_ID = "UnitStatsWidget"
		self.screenWidgetData = {}		
		self.nWidgetCount = 0
		self.Z_BACKGROUND = -2.1
		self.Z_CONTROLS = self.Z_BACKGROUND - 0.2
		self.currentScreen = UNIT_STATISTICS
		self.EventKeyDown=6
		self.currentUnit=0
		global g_bShowAllPlayers
		
		# Load the Unit Statistics Mod Config INI file containing all of the configuration information		
		config = CvConfigParser.CvConfigParser("Unit Statistics Mod Config.ini")
		
		# If we actually were able to open the "Unit Statistics Mod Config.ini" file then read in the values.
		# otherwise we'll keep the default values that were set at the top of this file.
		if(config != None):
			g_bShowAllPlayers = config.getboolean("Unit Statistics Mod", "Show All Players", false)


	# This function is called from 'outside', namely from the event manager. 
	# strScreen is the argument that defines which screen is to be shown first
	def	startScreen(self, objUnit, strScreen):
		self.currentUnit = objUnit

		if (strScreen == "unit"):
			self.currentScreen = UNIT_STATISTICS
		if (strScreen == "player"):
			self.currentScreen = PLAYER_STATISTICS
		if (strScreen == "highscores"):
			self.currentScreen = HIGHSCORES
			
		self.interfaceScreen()	

		
	"Statistics Screen"
	def interfaceScreen(self):
		
		screen = self.getScreen()
		
		# Get the screen height, width etc.
		self.calculateScreenWidgetData(screen)

		objUnit = self.currentUnit
		
		screen.showScreen(PopupStates.POPUPSTATE_IMMEDIATE, False)

		
		# draws the top and bottom panel of the screen
		self.drawScreenTop(screen)
		self.drawScreenBottom(screen)
		
		# Draws the main part of the screen (depending on which screen is to be drawn)
		if(self.currentScreen == UNIT_STATISTICS):
			if (objUnit != None):
				self.drawUnitScreen(screen, objUnit)
			else:
				self.drawPlayerScreen(screen)
		elif(self.currentScreen == PLAYER_STATISTICS):
			self.drawPlayerScreen(screen)
		elif(self.currentScreen == HIGHSCORES):
			self.drawHighScoresScreen(screen)


	# Returns the instance of the unit statistics screen.						
	def getScreen(self):
		if (self.currentScreen == HIGHSCORES):	
			return CyGInterfaceScreen("HighScoresScreen", CvScreenEnums.HIGHSCORES_SCREEN)
		elif (self.currentScreen == UNIT_STATISTICS):
			return CyGInterfaceScreen("UnitStatisticsScreen", CvScreenEnums.UNITSTATS_SCREEN)
		elif (self.currentScreen == PLAYER_STATISTICS):
			return CyGInterfaceScreen("PlayerStatisticsScreen", CvScreenEnums.PLAYERSTATS_SCREEN)
		else:
			return False
		
	def drawUnitScreen(self, screen, objUnit):	

		# Defines the structure of the screen
		screen.addPanel(UNIT_INFORMATION_PANEL_ID, u"", u"", True, False, self.screenWidgetData[UNIT_INFORMATION_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN)
		screen.addPanel(UNIT_INFORMATION_INNER_PANEL_ID, u"", u"", True, False, self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_EMPTY)
		screen.addPanel(UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_ID, u"", u"", True, False, self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN )
		screen.setText(UNIT_INFORMATION_TEXT_PANEL_ID, "Background", self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL], CvUtil.FONT_CENTER_JUSTIFY, self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		
		screen.addPanel(UNIT_INFORMATION_DETAILS_PANEL_ID, u"", u"", True, False, self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_EMPTY)
		screen.addPanel(UNIT_INFORMATION_PROMOTION_PANEL_ID, u"", u"", True, False, self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN)
		screen.addPanel(UNIT_INFORMATION_INNER_PROMOTION_PANEL_ID, localText.getText("TXT_KEY_WB_PROMOTIONS", ()), u"", False, False, self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_EMPTY)
		screen.addPanel(UNIT_INFORMATION_HIGHSCORE_PANEL_ID, u"", u"", True, False, self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN)
		screen.addPanel(UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_ID + "1", localText.getText("TXT_KEY_HIGHSCORES_SMALL", ()), u"", False, False, self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_HEIGHT] / 2, PanelStyles.PANEL_STYLE_EMPTY)
		screen.addPanel(UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_ID + "2", u"", u"", False, True, self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_Y] + self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_HEIGHT] / 2, self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_HEIGHT] / 2, PanelStyles.PANEL_STYLE_EMPTY)
		screen.attachListBoxGFC( UNIT_INFORMATION_DETAILS_PANEL_ID, UNIT_INFORMATION_DETAILS_LIST_ID, "", TableStyles.TABLE_STYLE_EMPTY )
		screen.enableSelect(UNIT_INFORMATION_DETAILS_LIST_ID, False)
		screen.attachMultiListControlGFC(UNIT_INFORMATION_INNER_PROMOTION_PANEL_ID, UNIT_INFORMATION_PROMOTION_LIST_CONTROL_ID, "", 1, 64, 64, TableStyles.TABLE_STYLE_STANDARD)


		# Build the unit XP string
		strXP = u"%d/%d" %(objUnit.getExperience(), objUnit.experienceNeeded())

		# Build the unit stats string
		if (objUnit.getDomainType() == DomainTypes.DOMAIN_AIR):
			fUnitHealth = int(objUnit.airBaseCombatStr()) * float(objUnit.currHitPoints()) / float(objUnit.maxHitPoints())
			fMaxHealth = int(objUnit.airBaseCombatStr())
		else:
			fUnitHealth = int(objUnit.baseCombatStr()) * float(objUnit.currHitPoints()) / float(objUnit.maxHitPoints())
			fMaxHealth = int(objUnit.baseCombatStr())
			
		if ( (objUnit.movesLeft() % gc.getMOVE_DENOMINATOR()) > 0 ):
			iDenom = 1
		else:
			iDenom = 0
		iCurrMoves = ((objUnit.movesLeft() / gc.getMOVE_DENOMINATOR()) + iDenom )
		TurnsinService = localText.getText("TXT_KEY_TURNS_IN_SERVICE", ()) + " " + str(gc.getGame().getGameTurn() - sdObjectGetVal("UnitStats", objUnit, "StartTurn"))
		strStats = u"%.2f/%.2f%c    %d/%d%c" %(fUnitHealth, fMaxHealth, CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR),iCurrMoves, objUnit.baseMoves(),CyGame().getSymbolID(FontSymbols.MOVES_CHAR))


		# Show the unit information (Unit Type, Level, Health, Turns in service)
		screen.appendListBoxString( UNIT_INFORMATION_DETAILS_LIST_ID, objUnit.getNameNoDesc(), WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		screen.appendListBoxString( UNIT_INFORMATION_DETAILS_LIST_ID, localText.getText("TXT_KEY_UNITSTATS_UNITTYPE", ()) + " " + PyInfo.UnitInfo(objUnit.getUnitType()).getDescription(), WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		screen.appendListBoxString( UNIT_INFORMATION_DETAILS_LIST_ID, localText.getText("INTERFACE_PANE_LEVEL", ()) + " " + str(objUnit.getLevel()) + "     XP: " + strXP, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		screen.appendListBoxString( UNIT_INFORMATION_DETAILS_LIST_ID, "" + strStats + "\n", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		screen.appendListBoxString( UNIT_INFORMATION_DETAILS_LIST_ID, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		screen.appendListBoxString( UNIT_INFORMATION_DETAILS_LIST_ID, TurnsinService, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		screen.appendListBoxString( UNIT_INFORMATION_DETAILS_LIST_ID, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

				
		# Get the promotion list for the unit
		promotionList = []

		for i in range(gc.getNumPromotionInfos()):				
			# If the unit has the promotion add it to the promotion list
			if(objUnit.isHasPromotion(i)):
				promotionList.append(gc.getPromotionInfo(i))

		# Add all of the promotions the unit has.
		for promotion in promotionList:
			screen.appendMultiListButton( UNIT_INFORMATION_PROMOTION_LIST_CONTROL_ID, promotion.getButton(), 0, WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROMOTION, gc.getInfoTypeForString(promotion.getType()), -1, false )

		# Add all of the high scores the unit has.
		HSUnitList = objUnitStatisticsUtils.getHighScoresCurrentUnit(objUnit)
		HSList = objUnitStatisticsUtils.getAllHighScores()
		for highscore in HSUnitList:
			if (HSUnitList.index(highscore) < 7):
				screen.attachImageButton( UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_ID + "1", highscore + "_HSBox" + str(HSList.index(highscore)), 
										"Art/Interface/Buttons/Highscores/"+highscore+".dds", GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_GENERAL, -1, -1, False )
			else:	
				screen.attachImageButton( UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_ID + "2", highscore + "_HSBox" + str(HSList.index(highscore)), 
										"Art/Interface/Buttons/Highscores/"+highscore+".dds", GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_GENERAL, -1, -1, False )

		screen.addUnitGraphicGFC(UNIT_GRAPHIC, objUnit.getUnitType(), self.screenWidgetData[UNIT_ANIMATION_X], self.screenWidgetData[UNIT_ANIMATION_Y], self.screenWidgetData[UNIT_ANIMATION_WIDTH], self.screenWidgetData[UNIT_ANIMATION_HEIGHT], WidgetTypes.WIDGET_GENERAL, -1, -1, self.screenWidgetData[UNIT_ANIMATION_ROTATION_X], self.screenWidgetData[UNIT_ANIMATION_ROTATION_Z], self.screenWidgetData[UNIT_ANIMATION_SCALE], True)
		screen.attachMultilineText(UNIT_INFORMATION_INNER_PANEL_ID, "Stats", objUnitStatisticsUtils.getUnitStatisticsString(objUnit), WidgetTypes.WIDGET_GENERAL, -1, -1, -1)

	def drawPlayerScreen(self, screen):	

		# Defines the structure of the screen
		screen.addPanel(UNIT_INFORMATION_PANEL_ID, u"", u"", True, True, self.screenWidgetData[UNIT_INFORMATION_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN)
		screen.addPanel(PLAYER_INFORMATION_INNER_PANEL_ID, u"", u"", True, True, self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_X], self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_Y], self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_WIDTH], self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_EMPTY)
		screen.addPanel(UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_ID, u"", u"", True, False, self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_Y], self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_WIDTH], self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN )
		screen.setText(UNIT_INFORMATION_TEXT_PANEL_ID, "Background", self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL2], CvUtil.FONT_CENTER_JUSTIFY, self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL_X], self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.attachMultilineText(PLAYER_INFORMATION_INNER_PANEL_ID, "Stats", objUnitStatisticsUtils.getPlayerStatisticsString(), WidgetTypes.WIDGET_GENERAL, -1, -1, -1)
		screen.bringMinimapToFront()


	def drawHighScoresScreen(self, screen):	

		# Get the list of all units with at least one high score
		HSUnitList = objUnitStatisticsUtils.getHighScoreUnits()
		
		# Define the height of the lefthand list depending on the number of highscore units
		HEIGHT = min(len(HSUnitList) * 72 + 70, self.screenWidgetData[HIGHSCORE_LIST_PANEL_HEIGHT])
		HEIGHT2 = HEIGHT - (14*self.screenWidgetData[BORDER_WIDTH])

		# Defines the structure of the screen
		screen.addPanel(HIGHSCORE_LIST_PANEL_ID, u"", u"", True, False, self.screenWidgetData[HIGHSCORE_LIST_PANEL_X], self.screenWidgetData[HIGHSCORE_LIST_PANEL_Y], self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH], HEIGHT, PanelStyles.PANEL_STYLE_MAIN)
		screen.addPanel(HIGHSCORE_LIST_INNER_PANEL_ID, u"", u"", True, True, self.screenWidgetData[HIGHSCORE_LIST_INNER_PANEL_X], self.screenWidgetData[HIGHSCORE_LIST_INNER_PANEL_Y], self.screenWidgetData[HIGHSCORE_LIST_INNER_PANEL_WIDTH], HEIGHT2, PanelStyles.PANEL_STYLE_IN)
		screen.addPanel(HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_ID, u"", u"", True, False, self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_X], self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_Y], self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_WIDTH], self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN )
		screen.setText(HIGHSCORE_LIST_TEXT_PANEL_ID, "Background", self.screenWidgetData[HIGHSCORE_LIST_TEXT_PANEL], CvUtil.FONT_CENTER_JUSTIFY, self.screenWidgetData[HIGHSCORE_LIST_TEXT_PANEL_X], self.screenWidgetData[HIGHSCORE_LIST_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				
		screen.addPanel(HIGHSCORE_PAGE_PANEL_ID, u"", u"", True, False, self.screenWidgetData[HIGHSCORE_PAGE_PANEL_X], self.screenWidgetData[HIGHSCORE_PAGE_PANEL_Y], self.screenWidgetData[HIGHSCORE_PAGE_PANEL_WIDTH], self.screenWidgetData[HIGHSCORE_PAGE_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN)
		screen.addPanel(HIGHSCORE_PAGE_INNER_PANEL_ID, u"", u"", True, True, self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_X], self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_Y], self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_WIDTH], self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_IN)
		screen.addPanel(HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_ID, u"", u"", True, False, self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_X], self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_Y], self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_WIDTH], self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_MAIN )
		screen.setText(HIGHSCORE_PAGE_TEXT_PANEL_ID, "Background", self.screenWidgetData[HIGHSCORE_PAGE_TEXT_PANEL], CvUtil.FONT_CENTER_JUSTIFY, self.screenWidgetData[HIGHSCORE_PAGE_TEXT_PANEL_X], self.screenWidgetData[HIGHSCORE_PAGE_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		self.drawMiniMap(screen)

		# For each unit with at least one highscore, a little panel is added
		for unit in HSUnitList:
			UnitID = self.numberToAlpha(unit.getID())+ "-" + self.numberToAlpha(unit.getOwner())
			screen.attachPanel(HIGHSCORE_LIST_INNER_PANEL_ID, UnitID, u"", u"", False, False, PanelStyles.PANEL_STYLE_DAWN)

			# Adds a button to the unit's panel
			screen.attachImageButton(UnitID, UnitID + "_UnitButton0", 
										gc.getUnitInfo(unit.getUnitType()).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_GENERAL, -1, -1, False )

			# Some structural panels to make it look right
			screen.attachPanel(UnitID, UnitID+"RightPanel", "", "", False, False, PanelStyles.PANEL_STYLE_EMPTY)
			screen.attachPanel(UnitID+"RightPanel", UnitID+"Buttons","", "", True, False, PanelStyles.PANEL_STYLE_EMPTY)

			# Add a 'find' button. Clicking on it closes the screen and selects the unit (see handleinput if you want to change this)
			screen.attachImageButton( UnitID, UnitID+"_FindButton1", 
										"Art/Interface/Buttons/Actions/Wake.dds", GenericButtonSizes.BUTTON_SIZE_32, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1, False )

			# Add all of the high scores the unit has.		
			screen.attachMultiListControlGFC(UnitID+"Buttons", UnitID+"Highscore", "", 1, 24, 24, TableStyles.TABLE_STYLE_STANDARD)
			for highscore in objUnitStatisticsUtils.getHighScoresCurrentUnit(unit):
				screen.appendMultiListButton( UnitID+"Highscore", "Art/Interface/Buttons/Highscores/"+highscore+".dds", 0, WidgetTypes.WIDGET_GENERAL, -1, -1, false )
			if (unit.getOwner() != gc.getGame().getActivePlayer() and not g_bShowAllPlayers):
				screen.enable(UnitID + "_UnitButton0", False)
				screen.enable(UnitID + "_FindButton1", False)
				screen.enable(UnitID+"Highscore", False)


		# This draws the righthand part of the screen
		i = 0
		for highscore in objUnitStatisticsUtils.getAllHighScores():
			i += 1
						
			# highscore 1, 3, 5 etc. are shown on the left...
			if (i % 2 == 0):
				screen.attachPanel("highscore" + str(i - 1) + "1", highscore, "", "", False, False, PanelStyles.PANEL_STYLE_DAWN)

			# ... 2, 4, 6 etc. on the right.	
			else:
				screen.attachPanel(HIGHSCORE_PAGE_INNER_PANEL_ID, "highscore" + str(i) + "1", "", "", False, False, PanelStyles.PANEL_STYLE_EMPTY)
				screen.attachPanel("highscore" + str(i) + "1", highscore, "", "", False, False, PanelStyles.PANEL_STYLE_DAWN)

			# Adds the button for the highscore	
			screen.attachImageButton( highscore, highscore + "_HighScoreButton", 
										"Art/Interface/Buttons/Highscores/"+highscore+".dds", GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_GENERAL, -1, -1, False )

			# disable the button if there is no unit holding that high score
			if (isinstance(objUnitStatisticsUtils.getHSUnit(highscore), (int))):
				screen.enable(highscore + "_HighScoreButton", False)

			# get the string with the current and hall of fame high scores and add it to the screen
			strHighscore = objUnitStatisticsUtils.getHSString(highscore, "player")
			screen.attachPanel(highscore, highscore+"Text", "", "", True, False, PanelStyles.PANEL_STYLE_EMPTY)
			screen.attachMultilineText(highscore+"Text", highscore+"Text1", strHighscore, WidgetTypes.WIDGET_GENERAL, -1, -1, -1)

	# Draws the minimap for the "High Scores" screen		
	def drawMiniMap(self, screen):	

		
		# Get the list of all units with at least one high score
		HSUnitList = objUnitStatisticsUtils.getHighScoreUnits()
		
		# Create a minimap
		screen.addPanel("MiniMapPanel", u"", "", False, False, self.screenWidgetData[MINIMAP_X], self.screenWidgetData[MINIMAP_Y], self.screenWidgetData[MINIMAP_WIDTH], self.screenWidgetData[MINIMAP_HEIGHT], PanelStyles.PANEL_STYLE_MAIN)
		screen.initMinimap(self.screenWidgetData[MINIMAP_X] + self.screenWidgetData[MINIMAP_BORDER], self.screenWidgetData[MINIMAP_X] + self.screenWidgetData[MINIMAP_WIDTH] - self.screenWidgetData[MINIMAP_BORDER], self.screenWidgetData[MINIMAP_Y] + self.screenWidgetData[MINIMAP_BORDER], self.screenWidgetData[MINIMAP_Y] + self.screenWidgetData[MINIMAP_HEIGHT] - self.screenWidgetData[MINIMAP_BORDER], self.Z_CONTROLS)
		screen.updateMinimapSection(False, False)

		screen.updateMinimapColorFromMap(MinimapModeTypes.MINIMAPMODE_MILITARY, 0.3)

		screen.setMinimapMode(MinimapModeTypes.MINIMAPMODE_MILITARY)
		screen.bringMinimapToFront()
		screen.updateMinimapVisibility()

	
		# Show all high score units on the minimap
		for loopUnit in HSUnitList:
			if (loopUnit.getOwner() != gc.getGame().getActivePlayer() and not g_bShowAllPlayers):
				continue
			else:
				iPlayer = loopUnit.getOwner()
				player = PyPlayer(iPlayer)
				iColor = gc.getPlayerColorInfo(gc.getPlayer(iPlayer).getPlayerColor()).getColorTypePrimary()
				screen.setMinimapColor(MinimapModeTypes.MINIMAPMODE_MILITARY, loopUnit.getX(), loopUnit.getY(), iColor, 0.6)						


	
	# Draws the top bar of the "Unit Statistics" screens
	def drawScreenTop(self, screen):
		screen.setDimensions(0, 0, self.screenWidgetData[SCREEN_WIDTH], self.screenWidgetData[SCREEN_HEIGHT])
		screen.addDrawControl(BACKGROUND_ID, ArtFileMgr.getInterfaceArtInfo("SCREEN_BG_OPAQUE").getPath(), 0, 0, self.screenWidgetData[SCREEN_WIDTH], self.screenWidgetData[SCREEN_HEIGHT], WidgetTypes.WIDGET_GENERAL, -1, -1 )
 		screen.addDDSGFC(BACKGROUND_ID, ArtFileMgr.getInterfaceArtInfo("MAINMENU_SLIDESHOW_LOAD").getPath(), 0, 0, self.screenWidgetData[SCREEN_WIDTH], self.screenWidgetData[SCREEN_HEIGHT], WidgetTypes.WIDGET_GENERAL, -1, -1 )
		
		screen.addPanel(SCREEN_TITLE_PANEL_ID, u"", u"", True, False, self.screenWidgetData[SCREEN_TITLE_PANEL_X], self.screenWidgetData[SCREEN_TITLE_PANEL_Y], self.screenWidgetData[SCREEN_TITLE_PANEL_WIDTH], self.screenWidgetData[SCREEN_TITLE_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_TOPBAR )
		screen.setText(SCREEN_TITLE_TEXT_PANEL_ID, "Background", self.screenWidgetData[SCREEN_TITLE_TEXT_PANEL], CvUtil.FONT_CENTER_JUSTIFY, self.screenWidgetData[SCREEN_TITLE_TEXT_PANEL_X], self.screenWidgetData[SCREEN_TITLE_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)


	# Draws the bottom bar of the "Unit Statistics" screens
	def drawScreenBottom(self, screen):
		screen.addPanel(BOTTOM_PANEL_ID, u"", u"", True, True, self.screenWidgetData[BOTTOM_PANEL_X], self.screenWidgetData[BOTTOM_PANEL_Y], self.screenWidgetData[BOTTOM_PANEL_WIDTH], self.screenWidgetData[BOTTOM_PANEL_HEIGHT], PanelStyles.PANEL_STYLE_BOTTOMBAR )
		screen.setText(UNIT_STATISTICS_TEXT_PANEL_ID, "Background", self.screenWidgetData[UNIT_STATISTICS_TEXT_PANEL], CvUtil.FONT_LEFT_JUSTIFY, self.screenWidgetData[UNIT_STATISTICS_TEXT_PANEL_X], self.screenWidgetData[UNIT_STATISTICS_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(HIGHSCORES_TEXT_PANEL_ID, "Background", self.screenWidgetData[HIGHSCORES_TEXT_PANEL], CvUtil.FONT_LEFT_JUSTIFY, self.screenWidgetData[HIGHSCORES_TEXT_PANEL_X], self.screenWidgetData[HIGHSCORES_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(PLAYER_STATISTICS_TEXT_PANEL_ID, "Background", self.screenWidgetData[PLAYER_STATISTICS_TEXT_PANEL], CvUtil.FONT_LEFT_JUSTIFY, self.screenWidgetData[PLAYER_STATISTICS_TEXT_PANEL_X], self.screenWidgetData[PLAYER_STATISTICS_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(EXIT_TEXT_PANEL_ID, "Background", self.screenWidgetData[EXIT_TEXT_PANEL], CvUtil.FONT_RIGHT_JUSTIFY, self.screenWidgetData[EXIT_TEXT_PANEL_X], self.screenWidgetData[EXIT_TEXT_PANEL_Y], self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1 )


	def closeScreen(self):
		screen = self.getScreen()
		screen.hideScreen()
		
	def hideScreen(self):
		screen = self.getScreen()
		screen.hideScreen()

	# handles the display of the units info pane
	def showHSInfoPane(self, id):

		screen = self.getScreen()

		self.calculateScreenWidgetData(screen)

		screen.addPanel( HIGHSCORE_INFO_PANE_ID, u"", u"", True, True, \
						self.screenWidgetData[HIGHSCORE_INFO_PANE_X], self.screenWidgetData[HIGHSCORE_INFO_PANE_Y] ,self.screenWidgetData[HIGHSCORE_INFO_PANE_WIDTH], self.screenWidgetData[HIGHSCORE_INFO_PANE_HEIGHT], \
						PanelStyles.PANEL_STYLE_HUD_HELP )


		HSList = objUnitStatisticsUtils.getAllHighScores()

		szText = objUnitStatisticsUtils.getHSString(HSList[id], "unit")

		szTextSized = "<font=2>"+ szText +"</font=2>"

		# create shadow text
		szTextBlack = localText.changeTextColor(szText, gc.getInfoTypeForString("COLOR_BLACK"))

		szTextBlackSized = "<font=2>"+ szTextBlack +"</font=2>"
		
		# display shadow text
		screen.addMultilineText( "UNIT_INFO_TEXT_SHADOW", szTextBlackSized, \
								self.screenWidgetData[HIGHSCORE_INFO_PANE_X] + 6, self.screenWidgetData[HIGHSCORE_INFO_PANE_Y] +6, \
								self.screenWidgetData[HIGHSCORE_INFO_PANE_WIDTH] -3, self.screenWidgetData[HIGHSCORE_INFO_PANE_HEIGHT] - 3, \
								WidgetTypes.WIDGET_GENERAL, -1, -1, \
								CvUtil.FONT_LEFT_JUSTIFY)
		# display text
		screen.addMultilineText( "UNIT_INFO_TEXT", szTextSized, \
								self.screenWidgetData[HIGHSCORE_INFO_PANE_X] + 5, self.screenWidgetData[HIGHSCORE_INFO_PANE_Y] +5, \
								self.screenWidgetData[HIGHSCORE_INFO_PANE_WIDTH] -3, self.screenWidgetData[HIGHSCORE_INFO_PANE_HEIGHT] - 3, \
								WidgetTypes.WIDGET_GENERAL, -1, -1, \
								CvUtil.FONT_LEFT_JUSTIFY)

	def hideHSInfoPane(self):
		screen = self.getScreen()
		screen.hide("UNIT_INFO_TEXT")
		screen.hide("UNIT_INFO_TEXT_SHADOW")
		screen.hide(HIGHSCORE_INFO_PANE_ID)

	# handles the display of the button info pane
	def showButtonInfoPane(self, id):

		screen = self.getScreen()

		self.calculateScreenWidgetData(screen)

		screen.addPanel( BUTTON_INFO_PANE_ID, u"", u"", True, True, \
						self.screenWidgetData[BUTTON_INFO_PANE_X], self.screenWidgetData[BUTTON_INFO_PANE_Y] ,self.screenWidgetData[BUTTON_INFO_PANE_WIDTH], self.screenWidgetData[BUTTON_INFO_PANE_HEIGHT], \
						PanelStyles.PANEL_STYLE_HUD_HELP )

		if (id == 0):
			szText = localText.getText("TXT_KEY_UNITSTATS_UNITBUTTON", ())
		elif (id == 1):
			szText = localText.getText("TXT_KEY_UNITSTATS_FINDBUTTON", ())

		szTextSized = "<font=2>"+ szText +"</font=2>"

		# create shadow text
		szTextBlack = localText.changeTextColor(szText, gc.getInfoTypeForString("COLOR_BLACK"))

		szTextBlackSized = "<font=2>"+ szTextBlack +"</font=2>"
		
		# display shadow text
		screen.addMultilineText( "UNIT_INFO_TEXT_SHADOW", szTextBlackSized, \
								self.screenWidgetData[BUTTON_INFO_PANE_X] + 6, self.screenWidgetData[BUTTON_INFO_PANE_Y] +6, \
								self.screenWidgetData[BUTTON_INFO_PANE_WIDTH] -3, self.screenWidgetData[BUTTON_INFO_PANE_HEIGHT] - 3, \
								WidgetTypes.WIDGET_GENERAL, -1, -1, \
								CvUtil.FONT_LEFT_JUSTIFY)
		# display text
		screen.addMultilineText( "UNIT_INFO_TEXT", szTextSized, \
								self.screenWidgetData[BUTTON_INFO_PANE_X] + 5, self.screenWidgetData[BUTTON_INFO_PANE_Y] +5, \
								self.screenWidgetData[BUTTON_INFO_PANE_WIDTH] -3, self.screenWidgetData[BUTTON_INFO_PANE_HEIGHT] - 3, \
								WidgetTypes.WIDGET_GENERAL, -1, -1, \
								CvUtil.FONT_LEFT_JUSTIFY)

	def hideButtonInfoPane(self):
		screen = self.getScreen()
		screen.hide("UNIT_INFO_TEXT")
		screen.hide("UNIT_INFO_TEXT_SHADOW")
		screen.hide(BUTTON_INFO_PANE_ID)

	# If you hover your mouse over a unit button, its location on the map begins to blink	
	def setFlashPlot (self, screen, strName, strOnOrOff):
	
		# Get the unit object
		strUnitID, strPlayerID = strName.split("-")
		iPlayerID = self.alphaToNumber(strPlayerID)
		iUnitID = self.alphaToNumber(strUnitID)
		objUnit = gc.getPlayer(iPlayerID).getUnit(iUnitID)
		if (objUnit == None or isinstance(objUnit, (int))):
			return
		
		if (objUnit.getOwner() != gc.getGame().getActivePlayer() and not g_bShowAllPlayers):
			return
		
		# Get the color
		self.iActivePlayer = gc.getGame().getActivePlayer()
		iPlayer = objUnit.getOwner()
		player = PyPlayer(iPlayer)
		iColor = gc.getPlayerColorInfo(gc.getPlayer(iPlayer).getPlayerColor()).getColorTypePrimary()
						
		if (player.getTeam().isAtWar(gc.getPlayer(self.iActivePlayer).getTeam())):
			iColor = gc.getInfoTypeForString("COLOR_RED")
		elif (gc.getPlayer(iPlayer).getTeam() != gc.getPlayer(self.iActivePlayer).getTeam()):
			iColor = gc.getInfoTypeForString("COLOR_YELLOW")
		else:
			iColor = gc.getInfoTypeForString("COLOR_WHITE")

		# If the mouse is moving on the button, the unit's plot starts to blink
		if (strOnOrOff == "on"):		
			screen.minimapFlashPlot(objUnit.getX(), objUnit.getY(), iColor, -1)
			
		# If the mouse is moving off the button, the blinking stops	
		if (strOnOrOff == "off"):		
			screen.minimapClearAllFlashingTiles()

	def minimapClicked(self):
		self.hideScreen()
		self.currentScreen = UNIT_STATISTICS

	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		
		screen = self.getScreen()

		# Get the data
		theKey = int(inputClass.getData())
		self.currentUnit = CyInterface().getSelectionUnit(0)

		# If the escape key was pressed then set the current screen to unit statistics
		if (inputClass.getNotifyCode() == self.EventKeyDown and theKey == int(InputTypes.KB_ESCAPE)):
			self.currentScreen = UNIT_STATISTICS

		# If the exit text was pressed then set the current screen to unit statistics.
		if(inputClass.getFunctionName() == EXIT_TEXT_PANEL_ID):
			self.currentScreen = UNIT_STATISTICS
	
		# Handles the info panes that appear if you hover your mouse over a high score button on the unit screen.
		id = inputClass.getID()
		if ( inputClass.getNotifyCode() == NotifyCode.NOTIFY_CURSOR_MOVE_ON  ):
			if (inputClass.getFunctionName().endswith("HSBox")):
			    self.showHSInfoPane(id)
			elif (inputClass.getFunctionName().endswith("UnitButton")):
				strName, function = inputClass.getFunctionName().split("_")
				self.setFlashPlot(screen, strName, "on")
#				self.showButtonInfoPane(id)
#			elif (inputClass.getFunctionName().endswith("FindButton")):
#				self.showButtonInfoPane(id)
		elif ( inputClass.getNotifyCode() == NotifyCode.NOTIFY_CURSOR_MOVE_OFF ):
			if (inputClass.getFunctionName().endswith("HSBox")):
			    self.hideHSInfoPane()	
			elif (inputClass.getFunctionName().endswith("UnitButton")):
				strName, function = inputClass.getFunctionName().split("_")
				self.setFlashPlot(screen, strName, "off")
#				self.hideButtonInfoPane()
#			elif (inputClass.getFunctionName().endswith("FindButton")):
#			    self.hideButtonInfoPane()
			
		# If the player statistics text was pressed and we aren't currently looking 
		# at the player statistics screen then set the current screen to 
		# player statistics, hide the screen and redraw the screen.		
		if(inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED and inputClass.getFunctionName() == PLAYER_STATISTICS_TEXT_PANEL_ID and self.currentScreen != PLAYER_STATISTICS):
			self.currentScreen = PLAYER_STATISTICS
			self.hideScreen()
			self.interfaceScreen()
			return

		# If the unit statistics text was pressed and we aren't currently looking 
		# at the unit statistics screen then set the current screen to 
		# unit statistics, hide the screen and redraw the screen.		
		if(inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED and inputClass.getFunctionName() == UNIT_STATISTICS_TEXT_PANEL_ID and self.currentScreen != UNIT_STATISTICS):
			if (self.currentUnit == None or isinstance(self.currentUnit, (int))):
				return
			self.currentScreen = UNIT_STATISTICS
			self.hideScreen()
			self.interfaceScreen()
			return
		
		# If the highscores text was pressed and we aren't currently looking 
		# at the highscores screen then set the current screen to 
		# highscores, hide the screen and redraw the screen.		
		if(inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED and inputClass.getFunctionName() == HIGHSCORES_TEXT_PANEL_ID and self.currentScreen != HIGHSCORES):
			self.currentScreen = HIGHSCORES
			self.hideScreen()
			self.interfaceScreen()
			return


		# If someone pressed one of the buttons in the screen then handle the
		# action
		if(inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED and inputClass.getFunctionName().endswith("Button")):
			# Split up the function name into the unit ID string and the actual
			# action that was performed
			strName, function = inputClass.getFunctionName().split("_")
			
			# If the function was unit, then close the screen and find the unit
			if(inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED and function == "UnitButton"):
				strUnitID, strPlayerID = strName.split("-")
				iPlayerID = self.alphaToNumber(strPlayerID)
				iUnitID = self.alphaToNumber(strUnitID)
				objUnit = gc.getPlayer(iPlayerID).getUnit(iUnitID)
				if (objUnit == None or isinstance(objUnit, (int))):
					return
				if(objUnit.getOwner() != gc.getGame().getActivePlayer() and not g_bShowAllPlayers):
					return
				self.currentUnit = objUnit
				self.currentScreen = UNIT_STATISTICS
				self.hideScreen()
				self.interfaceScreen()

			# If the function was highscore, then open the unit screen for the corresponding unit
			if(inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED and function == "HighScoreButton"):

				objUnit = objUnitStatisticsUtils.getHSUnit(strName)
				if (objUnit == None or isinstance(objUnit, (int))):
					return
				if(objUnit.getOwner() != gc.getGame().getActivePlayer() and not g_bShowAllPlayers):
					return
				self.currentUnit = objUnit
				self.currentScreen = UNIT_STATISTICS
				self.hideScreen()
				self.interfaceScreen()

			# If the function was find, then close the screen and find the unit
			if(inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED and function == "FindButton"):
				
				strUnitID, strPlayerID = strName.split("-")
				iPlayerID = self.alphaToNumber(strPlayerID)
				iUnitID = self.alphaToNumber(strUnitID)
				objUnit = gc.getPlayer(iPlayerID).getUnit(iUnitID)

				# If the unit is not set to None then look at them and select
				# them.
				if(objUnit != None):
					CyCamera().JustLookAtPlot(objUnit.plot())
					if(not CyGame().isNetworkMultiPlayer()):
						CyInterface().selectUnit(objUnit, true, false, false)

				self.currentScreen = UNIT_STATISTICS
	
				return

		return 0

	def update(self, fDelta):
		return

	# returns a unique ID for a widget in this screen
	def getNextWidgetName(self):
		szName = self.WIDGET_ID + str(self.nWidgetCount)
		self.nWidgetCount += 1
		return szName

	# Calculates the screens widgets positions, dimensions, text, etc.
	def calculateScreenWidgetData(self, screen):
		' Calculates the screens widgets positions, dimensions, text, etc. '
		
		# The border width should not be a hard coded number
		self.screenWidgetData[BORDER_WIDTH] = 4
		
		self.screenWidgetData[SCREEN_WIDTH] = screen.getXResolution()
		self.screenWidgetData[SCREEN_HEIGHT] = screen.getYResolution()

			
		# Screen title panel information
		self.screenWidgetData[SCREEN_TITLE_PANEL_WIDTH] = self.screenWidgetData[SCREEN_WIDTH]
		self.screenWidgetData[SCREEN_TITLE_PANEL_HEIGHT] = 55
		self.screenWidgetData[SCREEN_TITLE_PANEL_X] = 0
		self.screenWidgetData[SCREEN_TITLE_PANEL_Y] = 0
		self.screenWidgetData[SCREEN_TITLE_TEXT_PANEL] = u"<font=4b>" + localText.getText("TXT_KEY_UNIT_STATISTICS", ()) + "</font>"
		self.screenWidgetData[SCREEN_TITLE_TEXT_PANEL_X] = self.screenWidgetData[SCREEN_WIDTH]/2
		self.screenWidgetData[SCREEN_TITLE_TEXT_PANEL_Y] = 8


		# Exit panel information		
		self.screenWidgetData[BOTTOM_PANEL_WIDTH] = self.screenWidgetData[SCREEN_WIDTH]
		self.screenWidgetData[BOTTOM_PANEL_HEIGHT] = 55
		self.screenWidgetData[BOTTOM_PANEL_X] = 0
		self.screenWidgetData[BOTTOM_PANEL_Y] = self.screenWidgetData[SCREEN_HEIGHT] - 55

		self.screenWidgetData[UNIT_STATISTICS_TEXT_PANEL] = u"<font=4>" + localText.getText("TXT_KEY_UNIT_STATISTICS", ())+ "</font>"
		self.screenWidgetData[UNIT_STATISTICS_TEXT_PANEL_X] = 30
		self.screenWidgetData[UNIT_STATISTICS_TEXT_PANEL_Y] = self.screenWidgetData[SCREEN_HEIGHT] - 42
		
		self.screenWidgetData[HIGHSCORES_TEXT_PANEL] = u"<font=4>" + localText.getText("TXT_KEY_HIGHSCORES", ()) + "</font>"
		self.screenWidgetData[HIGHSCORES_TEXT_PANEL_X] = self.screenWidgetData[SCREEN_WIDTH] / 4 + 30
		self.screenWidgetData[HIGHSCORES_TEXT_PANEL_Y] = self.screenWidgetData[SCREEN_HEIGHT] - 42

		self.screenWidgetData[PLAYER_STATISTICS_TEXT_PANEL] = u"<font=4>" + localText.getText("TXT_KEY_PLAYER_STATISTICS", ()) + "</font>"
		self.screenWidgetData[PLAYER_STATISTICS_TEXT_PANEL_X] = self.screenWidgetData[SCREEN_WIDTH] / 2 + 30
		self.screenWidgetData[PLAYER_STATISTICS_TEXT_PANEL_Y] = self.screenWidgetData[SCREEN_HEIGHT] - 42

		self.screenWidgetData[EXIT_TEXT_PANEL] = u"<font=4>" + localText.getText("TXT_KEY_UNITSTATS_EXIT", ()) + "</font>"
		self.screenWidgetData[EXIT_TEXT_PANEL_X] = self.screenWidgetData[SCREEN_WIDTH] - 30
		self.screenWidgetData[EXIT_TEXT_PANEL_Y] = self.screenWidgetData[SCREEN_HEIGHT] - 42
		

				
		# Unit screen information
		self.screenWidgetData[UNIT_INFORMATION_PANEL_X] = self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_PANEL_Y] = self.screenWidgetData[SCREEN_TITLE_PANEL_HEIGHT] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_PANEL_WIDTH] = self.screenWidgetData[SCREEN_WIDTH] - (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[UNIT_INFORMATION_PANEL_HEIGHT] = (self.screenWidgetData[SCREEN_HEIGHT] - ((self.screenWidgetData[BORDER_WIDTH]*2) + self.screenWidgetData[SCREEN_TITLE_PANEL_HEIGHT] + self.screenWidgetData[BOTTOM_PANEL_HEIGHT]))
		self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_X] = self.screenWidgetData[UNIT_INFORMATION_PANEL_X] + self.screenWidgetData[BORDER_WIDTH] + (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_Y] = self.screenWidgetData[UNIT_INFORMATION_PANEL_Y] + self.screenWidgetData[BORDER_WIDTH] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_WIDTH] = self.screenWidgetData[UNIT_INFORMATION_PANEL_WIDTH] - (self.screenWidgetData[BORDER_WIDTH]*6)
		self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_HEIGHT] = 30
		self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL_X] = self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_X] + (self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_WIDTH]/2)
		self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL_Y] = self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_Y] + 4

		self.screenWidgetData[UNIT_ANIMATION_X] = self.screenWidgetData[UNIT_INFORMATION_PANEL_X]+ self.screenWidgetData[BORDER_WIDTH]*4
		self.screenWidgetData[UNIT_ANIMATION_Y] = self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL_Y]+40
		self.screenWidgetData[UNIT_ANIMATION_WIDTH] = 303
		self.screenWidgetData[UNIT_ANIMATION_HEIGHT] = 200
		self.screenWidgetData[UNIT_ANIMATION_ROTATION_X] = -20
		self.screenWidgetData[UNIT_ANIMATION_ROTATION_Z] = 30
		self.screenWidgetData[UNIT_ANIMATION_SCALE] = 1.0

		self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_WIDTH] = 484
		self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_HEIGHT] = 128 + (self.screenWidgetData[BORDER_WIDTH]*9)		
		self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_X] = self.screenWidgetData[SCREEN_WIDTH] - self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_WIDTH] - (self.screenWidgetData[BORDER_WIDTH]*4)
		self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_Y] = self.screenWidgetData[UNIT_ANIMATION_Y] 

		self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_X] = self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_X] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_Y] = self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_Y] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_WIDTH] = self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_WIDTH] + (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[UNIT_INFORMATION_INNER_PROMOTION_PANEL_HEIGHT] = self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_HEIGHT] - (self.screenWidgetData[BORDER_WIDTH]*2)

		self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_X] = self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_X]
		self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_Y] = self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_Y] + self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_HEIGHT] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_WIDTH] = self.screenWidgetData[UNIT_INFORMATION_PROMOTION_PANEL_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_HEIGHT] = 148 + (self.screenWidgetData[BORDER_WIDTH]*9)

		self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_X] = self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_X] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_Y] = self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_Y] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_WIDTH] = self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_WIDTH] + (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_HEIGHT] = self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_HEIGHT] - (self.screenWidgetData[BORDER_WIDTH]*2)


		self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_X] = self.screenWidgetData[UNIT_INFORMATION_PANEL_X] + self.screenWidgetData[BORDER_WIDTH]*3
		self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_Y] = self.screenWidgetData[UNIT_ANIMATION_Y] + self.screenWidgetData[UNIT_ANIMATION_HEIGHT] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_WIDTH] = self.screenWidgetData[UNIT_INFORMATION_PANEL_WIDTH] - (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[UNIT_INFORMATION_INNER_PANEL_HEIGHT] = self.screenWidgetData[UNIT_INFORMATION_PANEL_HEIGHT] - self.screenWidgetData[BORDER_WIDTH] - self.screenWidgetData[UNIT_ANIMATION_Y] - self.screenWidgetData[UNIT_ANIMATION_HEIGHT]

		self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL] = "<font=3b>" + localText.getText("TXT_KEY_UNIT_OVERVIEW", ()) + "</font>"
		self.screenWidgetData[UNIT_INFORMATION_TEXT_PANEL2] = "<font=3b>" + localText.getText("TXT_KEY_PLAYER_OVERVIEW", ()) + "</font>"
		self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_X] = self.screenWidgetData[UNIT_ANIMATION_X] + self.screenWidgetData[UNIT_ANIMATION_WIDTH] + (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_Y] = self.screenWidgetData[UNIT_ANIMATION_Y]
		self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_WIDTH] = self.screenWidgetData[SCREEN_WIDTH] - (self.screenWidgetData[UNIT_ANIMATION_X] + self.screenWidgetData[UNIT_ANIMATION_WIDTH] + (self.screenWidgetData[BORDER_WIDTH])*6)
		self.screenWidgetData[UNIT_INFORMATION_DETAILS_PANEL_HEIGHT] = self.screenWidgetData[UNIT_ANIMATION_HEIGHT]

		# Player screen information
		self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_X] = self.screenWidgetData[UNIT_INFORMATION_PANEL_X] + self.screenWidgetData[BORDER_WIDTH]*3
		self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_Y] = self.screenWidgetData[UNIT_INFORMATION_PANEL_Y] + self.screenWidgetData[BORDER_WIDTH]*4 + self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_HEIGHT]
		self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_WIDTH] = self.screenWidgetData[UNIT_INFORMATION_PANEL_WIDTH] - (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[PLAYER_INFORMATION_INNER_PANEL_HEIGHT] = self.screenWidgetData[UNIT_INFORMATION_PANEL_HEIGHT] - (self.screenWidgetData[BORDER_WIDTH]*9) - self.screenWidgetData[UNIT_INFORMATION_TEXT_BACKGROUND_PANEL_HEIGHT]

		# High Score screen information
		self.screenWidgetData[HIGHSCORE_LIST_PANEL_X] = self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[HIGHSCORE_LIST_PANEL_Y] = self.screenWidgetData[SCREEN_TITLE_PANEL_HEIGHT] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH] = 300
		self.screenWidgetData[HIGHSCORE_LIST_INNER_PANEL_X] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_X] + (4*self.screenWidgetData[BORDER_WIDTH])
		self.screenWidgetData[HIGHSCORE_LIST_INNER_PANEL_Y] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_Y] + (10*self.screenWidgetData[BORDER_WIDTH])
		self.screenWidgetData[HIGHSCORE_LIST_INNER_PANEL_WIDTH] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH] - (8*self.screenWidgetData[BORDER_WIDTH])
		self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_X] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_X] + self.screenWidgetData[BORDER_WIDTH] + (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_Y] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_Y] + self.screenWidgetData[BORDER_WIDTH] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_WIDTH] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH] - (self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_X] + (self.screenWidgetData[BORDER_WIDTH]*2))
		self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_HEIGHT] = 30
		self.screenWidgetData[HIGHSCORE_LIST_TEXT_PANEL] = "<font=3b>" + localText.getText("TXT_KEY_HIGHSCORE_UNITS", ()) + "</font>"
		self.screenWidgetData[HIGHSCORE_LIST_TEXT_PANEL_X] = self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_X] + (self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_WIDTH]/2)
		self.screenWidgetData[HIGHSCORE_LIST_TEXT_PANEL_Y] = self.screenWidgetData[HIGHSCORE_LIST_TEXT_BACKGROUND_PANEL_Y] + 4		
		
		self.screenWidgetData[MINIMAP_X] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_X]
		self.screenWidgetData[MINIMAP_WIDTH] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH]
		self.screenWidgetData[MINIMAP_HEIGHT_MAX] = 300
		self.screenWidgetData[MINIMAP_BORDER] = 20
		self.screenWidgetData[MINIMAP_HEIGHT] = (self.screenWidgetData[MINIMAP_WIDTH] * CyMap().getGridHeight()) / CyMap().getGridWidth()
		if (self.screenWidgetData[MINIMAP_HEIGHT] > self.screenWidgetData[MINIMAP_HEIGHT_MAX]):
			self.screenWidgetData[MINIMAP_WIDTH] = (self.screenWidgetData[MINIMAP_HEIGHT_MAX] * CyMap().getGridWidth()) / CyMap().getGridHeight()
			self.screenWidgetData[MINIMAP_HEIGHT] = self.screenWidgetData[MINIMAP_HEIGHT_MAX]
		self.screenWidgetData[MINIMAP_Y] = (self.screenWidgetData[SCREEN_HEIGHT] - ((self.screenWidgetData[BORDER_WIDTH]*2) + self.screenWidgetData[BOTTOM_PANEL_HEIGHT])) - self.screenWidgetData[MINIMAP_HEIGHT]

		self.screenWidgetData[HIGHSCORE_LIST_PANEL_HEIGHT] = self.screenWidgetData[MINIMAP_Y] - (self.screenWidgetData[HIGHSCORE_LIST_PANEL_Y])
		self.screenWidgetData[HIGHSCORE_LIST_INNER_PANEL_HEIGHT] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_HEIGHT] - (14*self.screenWidgetData[BORDER_WIDTH])

		self.screenWidgetData[HIGHSCORE_PAGE_PANEL_X] = self.screenWidgetData[BORDER_WIDTH] + self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH]
		self.screenWidgetData[HIGHSCORE_PAGE_PANEL_Y] = self.screenWidgetData[SCREEN_TITLE_PANEL_HEIGHT] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[HIGHSCORE_PAGE_PANEL_WIDTH] = self.screenWidgetData[SCREEN_WIDTH] - self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH] - self.screenWidgetData[BORDER_WIDTH]*2
		self.screenWidgetData[HIGHSCORE_PAGE_PANEL_HEIGHT] = (self.screenWidgetData[SCREEN_HEIGHT] - ((self.screenWidgetData[BORDER_WIDTH]*3) + self.screenWidgetData[SCREEN_TITLE_PANEL_HEIGHT] + self.screenWidgetData[BOTTOM_PANEL_HEIGHT]))
		self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_X] = self.screenWidgetData[HIGHSCORE_PAGE_PANEL_X] + (4*self.screenWidgetData[BORDER_WIDTH])
		self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_Y] = self.screenWidgetData[HIGHSCORE_PAGE_PANEL_Y] + (10*self.screenWidgetData[BORDER_WIDTH])
		self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_WIDTH] = self.screenWidgetData[HIGHSCORE_PAGE_PANEL_WIDTH] - (8*self.screenWidgetData[BORDER_WIDTH])
		self.screenWidgetData[HIGHSCORE_PAGE_INNER_PANEL_HEIGHT] = self.screenWidgetData[HIGHSCORE_PAGE_PANEL_HEIGHT] - (14*self.screenWidgetData[BORDER_WIDTH])
		self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_X] = self.screenWidgetData[HIGHSCORE_PAGE_PANEL_X] + self.screenWidgetData[BORDER_WIDTH] + (self.screenWidgetData[BORDER_WIDTH]*2)
		self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_Y] = self.screenWidgetData[HIGHSCORE_PAGE_PANEL_Y] + self.screenWidgetData[BORDER_WIDTH] + self.screenWidgetData[BORDER_WIDTH]
		self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_WIDTH] = self.screenWidgetData[HIGHSCORE_PAGE_PANEL_WIDTH] - self.screenWidgetData[BORDER_WIDTH]*6
		self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_HEIGHT] = 30
		self.screenWidgetData[HIGHSCORE_PAGE_TEXT_PANEL] = "<font=3b>" + localText.getText("TXT_KEY_HIGHSCORES_SMALL", ()) + "</font>"
		self.screenWidgetData[HIGHSCORE_PAGE_TEXT_PANEL_X] = self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_X] + (self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_WIDTH]/2)
		self.screenWidgetData[HIGHSCORE_PAGE_TEXT_PANEL_Y] = self.screenWidgetData[HIGHSCORE_PAGE_TEXT_BACKGROUND_PANEL_Y] + 4		

		self.screenWidgetData[HIGHSCORE_INFO_PANE_WIDTH] = 250
		self.screenWidgetData[HIGHSCORE_INFO_PANE_HEIGHT] = 69
		self.screenWidgetData[HIGHSCORE_INFO_PANE_X] = self.screenWidgetData[UNIT_INFORMATION_HIGHSCORE_PANEL_X] - self.screenWidgetData[HIGHSCORE_INFO_PANE_WIDTH]
		self.screenWidgetData[HIGHSCORE_INFO_PANE_Y] = self.screenWidgetData[UNIT_INFORMATION_INNER_HIGHSCORE_PANEL_Y] + self.screenWidgetData[BORDER_WIDTH]*5

		self.screenWidgetData[BUTTON_INFO_PANE_WIDTH] = 250
		self.screenWidgetData[BUTTON_INFO_PANE_HEIGHT] = 69
		self.screenWidgetData[BUTTON_INFO_PANE_X] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_X] + self.screenWidgetData[HIGHSCORE_LIST_PANEL_WIDTH]
		self.screenWidgetData[BUTTON_INFO_PANE_Y] = self.screenWidgetData[HIGHSCORE_LIST_PANEL_Y]



	# Converts a number into its string representation. This is needed since
	# for whatever reason, numbers did not work very well when using them 
	# for all of the different panels in the screen. The
	# unit ID number 382343 is converted to: CHBCDC.
	def numberToAlpha(self, iNum):
		#             1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19  20  21  22  23  24  25  26
		alphaList = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z']
		strNum = str(iNum)
		strAlpha = ""
		
		# Go though the alphaList and convert the numbers to letters
		for i in range (len(strNum)):
			strAlpha = strAlpha + alphaList[int(strNum[i])]
			
		return strAlpha
	
	
	# Converts a number into its string representation. This is needed since
	# for whatever reason, numbers did not work very well when using them 
	# for all of the different panels in the screen. The
	# string "CHBCDC" is converted to: 382343.
	def alphaToNumber(self, strAlpha):
		#             1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19  20  21  22  23  24  25  26
		alphaList = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z']
		
		strNum = ""

		# Go though the alphaList and convert the letters to numbers
		for i in range (len(strAlpha)):
			strNum = strNum + str(alphaList.index(strAlpha[i]))
		
		return int(strNum)