## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005

from CvPythonExtensions import *
import CvUtil
import CvEventManager
import sys
import PyHelpers
import UnitStatisticsUtils
import CvConfigParser
import SdToolKitAdvanced
import CvStatisticsScreen


# globals
###################################################

gc = CyGlobalContext()

objUnitStatisticsUtils = UnitStatisticsUtils.UnitStatisticsUtils()


# Change this to either enable or disable unit promotion tracking.
# Default value is true
g_bTrackUnitPromotions = true

# Change this to either enable or disable goody received tracking.
# Default value is true
g_bTrackGoodyReceived = true

# Change this to either enable or disable movement tracking.
# Default value is true
g_bTrackMovement = true

# Change this to either enable or disable turn information tracking (amount of turns fortified)
# Default value is true
g_bTrackTurnInformation = true

# Change this to either enable or disable unit statistics for non-combatants (settlers, workers, missionaries, great people)
# Default value is true
g_bTrackNonCombatants = true

# Change this to either enable or disable unit statistics for all players (required for hotseat games)
# Default value is false
g_bTrackAllPlayers = false

# Change this to either enable or disable the display of other players' unit statistics 
# Leave disabled if you don't want any additional information about enemy units
# Default value is false
g_bShowAllPlayers = false

# Change this for different name generators.
# 0 = no unit naming (use this only when you also use the Unit Statistics CvGameCoreDLL!)
# 1 = Warrior 1, Warrior 2, Axeman 1, Axeman 2
# 2 = Random Name Generator by TheLopez
# 3 = Nexus naming 1st Army 1st Corp 1st Div
# Default value is 1
g_iUnitNaming = 1


STARTTURN = "StartTurn"
LOSSCOUNT = "LossCount"
DAMAGETYPE = ["DamageInflicted", 
			  "DamageInflictedAttacking", 
			  "DamageInflictedDefending", 
			  "DamageSuffered", 
			  "DamageSufferedAttacking", 
			  "DamageSufferedDefending",
			  "CollateralDamageInflicted",
			  "CollateralDamageSuffered",
			  "AirStrikeDamageInflicted",
			  "AirStrikeDamageSuffered"]


class CvUnitStatisticsEventManager:
	def __init__(self, eventManager):
	
		#################### ON EVENT MAP ######################
		self.EventKeyDown=6
		self.EventKeyUp=7
		self.eventManager = eventManager

		global g_bTrackUnitPromotions
		global g_bTrackGoodyReceived
		global g_bTrackNonCombatants 
		global g_bTrackMovement
		global g_bTrackAllPlayers
		global g_bTrackTurnInformation
		global g_iUnitNaming
		global g_bShowAllPlayers
		
		# Load the Unit Statistics Mod Config INI file containing all of the configuration information		
		config = CvConfigParser.CvConfigParser("Unit Statistics Mod Config.ini")
		
		# If we actually were able to open the "Unit Statistics Mod Config.ini" file then read in the values.
		# otherwise we'll keep the default values that were set at the top of this file.
		if(config != None):
			g_bTrackUnitPromotions = config.getboolean("Unit Statistics Mod", "Track Unit Promotions", true)
			g_bTrackGoodyReceived = config.getboolean("Unit Statistics Mod", "Track Goody Received", true)
			g_bTrackNonCombatants = config.getboolean("Unit Statistics Mod", "Track Non-Combat Units", true)
			g_bTrackAllPlayers = config.getboolean("Unit Statistics Mod", "Track All Players", false)
			g_bShowAllPlayers = config.getboolean("Unit Statistics Mod", "Show All Players", false)
			g_bTrackMovement = config.getboolean("Unit Statistics Mod", "Track Unit Movement", true)
			g_bTrackTurnInformation = config.getboolean("Unit Statistics Mod", "Track Turn Information", true)
			g_iUnitNaming = config.getint("Unit Statistics Mod", "Unit Naming", 1)

		eventManager.addEventHandler("kbdEvent",self.onKbdEvent)
		eventManager.addEventHandler("combatResult",self.onCombatResult)
		eventManager.addEventHandler("unitBuilt",self.onUnitBuilt)
		eventManager.addEventHandler("unitMove",self.onUnitMove)
		eventManager.addEventHandler("unitSetXY",self.onUnitSetXY)
		eventManager.addEventHandler("unitCreated",self.onUnitCreated)
		eventManager.addEventHandler("unitPromoted",self.onUnitPromoted)
		eventManager.addEventHandler("unitLost",self.onUnitLost)
		eventManager.addEventHandler("unitKilled",self.onUnitKilled)
		eventManager.addEventHandler("goodyReceived",self.onGoodyReceived)
		eventManager.addEventHandler("BeginPlayerTurn",self.onBeginPlayerTurn)
#		eventManager.addEventHandler("cityAcquired",self.onCityAcquired)
 		eventManager.setEventHandler("combatHit",self.onCombatHit)
		eventManager.setEventHandler("airIntercept", self.onAirIntercept)
		eventManager.setEventHandler("combatBegin",self.onCombatBegin)
		eventManager.setEventHandler("combatWithdrawal",self.onCombatWithdrawal)
		eventManager.setEventHandler("collateralHit", self.onCollateralHit)
		eventManager.setEventHandler("airStrikeHit", self.onAirStrikeHit)
		eventManager.setEventHandler("unitUpgraded",self.onUnitUpgraded)
			
	
	# Displays the unit statistics window if a unit is selected and the letter 
	# 'u' was pressed.
	def onKbdEvent(self, argsList):

		eventType,key,mx,my,px,py = argsList		
		theKey=int(key)

		# Try to get the selected unit
		objUnit = CyInterface().getSelectionUnit(0)

#		# Handle the event if the 'U' key was pressed 
#		if ( eventType == self.EventKeyDown and theKey == int(InputTypes.KB_U)):
#
#			# If we got a valid unit then launch the unit statistics popup.
#			if(objUnit != None and (objUnit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
#				objUnitStatisticsUtils.launchPopup(objUnit, "unit")
#			return 1
#		
#		# Handle the event if the 'X' key was pressed 
#		if ( eventType == self.EventKeyDown and theKey == int(InputTypes.KB_X)):
#			
#			# launch the player statistics popup.
#			objUnitStatisticsUtils.launchPopup(None, "player")
#			
#			return 1

		if (eventType == self.EventKeyDown and theKey == int(InputTypes.KB_U)):
			statisticsScreen = CvStatisticsScreen.CvStatisticsScreen()
			if (objUnit != None and (objUnit.getOwner() == gc.getGame().getActivePlayer() or g_bShowAllPlayers)):
				statisticsScreen.startScreen(objUnit, "unit")
				return 1
			else:
				statisticsScreen.startScreen(objUnit, "player")
				return 1

		return 0
		

	# Logs the unit combat results		
	def onCombatResult(self, argsList):

		pWinner,pLoser = argsList
		# If we got a valid unit log the unit location.
		if(pWinner != None and (pWinner.getOwner() == gc.getGame().getActivePlayer() or g_bTrackAllPlayers)):
			objUnitStatisticsUtils.logCombatResults(pWinner,pLoser)

	
	# Begins logging the unit information
	def onUnitBuilt(self, argsList):

		city = argsList[0]
		unit = argsList[1]
		iPlayerID = city.getOwner()

		# If we got a valid unit and city then begin logging the unit.

		if(unit != None and city != None and (g_bTrackAllPlayers or unit.getOwner() == gc.getGame().getActivePlayer()) and (unit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
			objUnitStatisticsUtils.beginLoggingUnit(city.getName(), unit, iPlayerID)
		
		
	def onUnitMove(self, argsList):
			
		pPlot,pUnit = argsList
		
		# If we got a valid unit log the unit move.

		if(g_bTrackMovement and pUnit != None and (g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()) and (pUnit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
			objUnitStatisticsUtils.logUnitMove(pUnit)
	

	def onUnitSetXY(self, argsList):
			
		pPlot,pUnit = argsList

		# If we got a valid unit log the unit location.

		if(g_bTrackMovement and pUnit != None and (g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()) and (pUnit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
			objUnitStatisticsUtils.logUnitLocation(pUnit)


	# Logs the unit created event information
	def onUnitCreated(self, argsList):

		pUnit = argsList[0]
		iPlayerID = pUnit.getOwner()
		
		# If we got a valid unit log the unit creation.

		if(pUnit != None and (g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()) and (pUnit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
			objUnitStatisticsUtils.logUnitCreation(pUnit, iPlayerID)

		# For non-combat-units, create a dummy-file in order to avoid problems with the high score calculation
		elif(pUnit != None and (g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()) and pUnit.getUnitCombatType() == -1):
			objUnitStatisticsUtils.setupUnitStats(pUnit)
			SdToolKitAdvanced.sdObjectSetVal("UnitStats", pUnit, STARTTURN, 6000)

		
	def onUnitLost(self, argsList):

		pUnit = argsList[0]
		if(pUnit != None and (g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()) and (pUnit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
			objUnitStatisticsUtils.onUnitLost(argsList)


	# Logs the unit promoted event information				
	def onUnitPromoted(self, argsList):

		pUnit, iPromotion = argsList

		# If we got a valid unit log the unit promotion.

		if(g_bTrackUnitPromotions and pUnit != None and (g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()) and (pUnit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
			objUnitStatisticsUtils.logUnitPromotion(pUnit, iPromotion)


	# Logs the goody received event information
	def onGoodyReceived(self, argsList):
		'Goody received'

		iPlayer, pPlot, pUnit, iGoodyType = argsList

		# If we got a valid unit log the unit promotion.

		if(g_bTrackGoodyReceived and pUnit != None and (g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()) and (pUnit.getUnitCombatType() != -1 or g_bTrackNonCombatants)):
			objUnitStatisticsUtils.logUnitGoodyReceived(pUnit, iGoodyType)

		
	def onCombatHit(self, argsList):

		genericArgs = argsList[0][0]
		attackerUnit = genericArgs[0]
		defenderUnit = genericArgs[1]
		iIsAttacker = genericArgs[2]
		iDamage = genericArgs[3]
		
		# If we got a valid unit log the unit damage.

		
		if(g_bTrackAllPlayers):
			objUnitStatisticsUtils.onCombatHit(argsList, 0)
		elif(attackerUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onCombatHit(argsList, 1)			
		elif(defenderUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onCombatHit(argsList, 2)

	def onAirIntercept(self, argsList):

		genericArgs = argsList[0][0]
		attackerUnit = genericArgs[0]
		interceptorUnit = genericArgs[1]
		iIsAttacker = genericArgs[2]
		iDamage = genericArgs[3]
		
		# If we got a valid unit log the unit damage.

		
		if(g_bTrackAllPlayers):
			objUnitStatisticsUtils.onAirIntercept(argsList, 0)
		elif(attackerUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onAirIntercept(argsList, 1)			
		elif(interceptorUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onAirIntercept(argsList, 2)

	def onCollateralHit(self, argsList):

		genericArgs = argsList[0][0]
		attackerUnit = genericArgs[0]
		defenderUnit = genericArgs[1]
		iDamage = genericArgs[2]
		
		# If we got a valid unit log the unit damage.

		
		if(g_bTrackAllPlayers):
			objUnitStatisticsUtils.onCollateralHit(argsList, 0)
		elif(attackerUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onCollateralHit(argsList, 1)			
		elif(defenderUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onCollateralHit(argsList, 2)
			
	def onAirStrikeHit(self, argsList):

		genericArgs = argsList[0][0]
		attackerUnit = genericArgs[0]
		defenderUnit = genericArgs[1]
		iDamage = genericArgs[2]
		
		# If we got a valid unit log the unit damage.

		
		if(g_bTrackAllPlayers):
			objUnitStatisticsUtils.onAirStrikeHit(argsList, 0)
		elif(attackerUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onAirStrikeHit(argsList, 1)			
		elif(defenderUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onAirStrikeHit(argsList, 2)

	def onCombatBegin(self, argsList):
	
		genericArgs = argsList[0][0]
		attackerUnit = genericArgs[0]
		defenderUnit = genericArgs[1]
		iDefenderOdds = genericArgs[2]
		
		# If we got a valid unit log the combat odds.
		if((g_bTrackAllPlayers or attackerUnit.getOwner() == gc.getGame().getActivePlayer() or defenderUnit.getOwner() == gc.getGame().getActivePlayer())):
			objUnitStatisticsUtils.onCombatBegin(argsList)
	
	def onCombatWithdrawal(self, argsList):
	
		genericArgs = argsList[0][0]
		attackerUnit = genericArgs[0]
		defenderUnit = genericArgs[1]
		
		# If we got a valid unit log the withdrawal
		
		if(g_bTrackAllPlayers):
			objUnitStatisticsUtils.onCombatWithdrawal(argsList, 0)
		elif(attackerUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onCombatWithdrawal(argsList, 1)			
		elif(defenderUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.onCombatWithdrawal(argsList, 2)

	def onUnitKilled(self, argsList):
	
		pUnit = argsList[0]
		if(g_bTrackAllPlayers or pUnit.getOwner() == gc.getGame().getActivePlayer()):
			for i in range(10):
				objUnitStatisticsUtils.checkHighScoresCurrentUnit([DAMAGETYPE[i]], pUnit)

	def onUnitUpgraded(self, argsList):
	
		genericArgs = argsList[0][0]
		pOldUnit = genericArgs[0]
		pNewUnit = genericArgs[1]
		
		if(g_bTrackAllPlayers or pOldUnit.getOwner() == gc.getGame().getActivePlayer() or pNewUnit.getOwner() == gc.getGame().getActivePlayer()):
			objUnitStatisticsUtils.logUnitUpgraded(argsList)
	
#	def onCityAcquired(self, argsList):
#		'City Acquired'
#		owner,playerType,city,bConquest,bTrade = argsList
#		print "city acquired test1"
#
#		if(bConquest and (playerType == gc.getGame().getActivePlayer() or g_bTrackAllPlayers)):
#			print "city acquired test2"
#			objUnitStatisticsUtils.logCityCapture(argsList)
		

	def onBeginPlayerTurn(self, argsList):

		iGameTurn, iPlayer = argsList
		if(g_bTrackTurnInformation and (g_bTrackAllPlayers or iPlayer == gc.getGame().getActivePlayer())):
			objUnitStatisticsUtils.logBeginPlayerTurn(argsList)